Package: topics
Type: Package
Title: Creating and Significance Testing Language Features for
        Visualisation
Version: 1.0
Authors@R: c(
    person("Leon", "Ackermann", 
    role = c("aut"), 
      comment = c(ORCID = "0009-0008-8583-8748"),
    ),
        person("Zhuojun", "Gu", 
        role = c("aut"), 
      comment = c(ORCID = "0009-0000-1610-4830"),
    ),
    person("Oscar", "Kjell", , "oscar.kjell@psy.lu.se", 
    role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0002-2728-6278"))
    )
Description: Implements differential language analysis with statistical tests and offers various language visualization techniques for n-grams and topics. It also supports the 'text' package. For more information, visit <https://r-topics.org/> and <https://www.r-text.org/>.
License: GPL-3
URL: https://r-topics.org/
BugReports: https://github.com/theharmonylab/topics/issues
Encoding: UTF-8
Archs: x64
SystemRequirements: Python (>= 3.6.0)
LazyData: true
BuildVignettes: true
Imports: textmineR, quanteda, ggplot2, dplyr, ggwordcloud, tibble,
        methods, readr, stopwords, Matrix, stringr, stringi, rlang,
        tidyr, purrr, data.table, ggforce, patchwork, utils
RoxygenNote: 7.3.3
Suggests: text, rJava, mallet, glmnet, knitr, rmarkdown, testthat,
        covr, vdiffr, httr, arrow
VignetteBuilder: knitr
Depends: R (>= 4.00)
NeedsCompilation: no
Packaged: 2026-06-13 05:21:05 UTC; oscarkjell
Author: Leon Ackermann [aut] (ORCID: <https://orcid.org/0009-0008-8583-8748>),
  Zhuojun Gu [aut] (ORCID: <https://orcid.org/0009-0000-1610-4830>),
  Oscar Kjell [aut, cre] (ORCID: <https://orcid.org/0000-0002-2728-6278>)
Maintainer: Oscar Kjell <oscar.kjell@psy.lu.se>
Repository: CRAN
Date/Publication: 2026-06-13 16:30:02 UTC
Built: R 4.4.3; ; 2026-06-13 18:47:20 UTC; unix
