# Generated by roxygen2: do not edit by hand

export(topicsAssess)
export(topicsClassify)
export(topicsDtm)
export(topicsDtmEval)
export(topicsGrams)
export(topicsGridLegend)
export(topicsModel)
export(topicsPlot)
export(topicsPlotOverview)
export(topicsPredict)
export(topicsPreds)
export(topicsScatterLegend)
export(topicsTest)
export(topicsTutorialData)
importFrom(Matrix,colSums)
importFrom(Matrix,t)
importFrom(data.table,":=")
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,contains)
importFrom(dplyr,desc)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_modify)
importFrom(dplyr,if_else)
importFrom(dplyr,join_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,slice_head)
importFrom(dplyr,summarise)
importFrom(dplyr,top_n)
importFrom(dplyr,ungroup)
importFrom(ggforce,geom_circle)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_identity)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_size_area)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(ggwordcloud,geom_text_wordcloud)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(patchwork,wrap_plots)
importFrom(purrr,map_dbl)
importFrom(quanteda,dfm)
importFrom(quanteda,dfm_select)
importFrom(quanteda,featnames)
importFrom(quanteda,quanteda_options)
importFrom(quanteda,tokens)
importFrom(quanteda,tokens_ngrams)
importFrom(quanteda,tokens_remove)
importFrom(readr,write_csv)
importFrom(rlang,"!!")
importFrom(rlang,sym)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stopwords,stopwords)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)
importFrom(stringr,str_split)
importFrom(textmineR,CalcGamma)
importFrom(textmineR,CreateDtm)
importFrom(textmineR,Dtm2Docs)
importFrom(tibble,as_tibble)
importFrom(tibble,is_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,gather)
importFrom(tidyr,separate)
importFrom(utils,combn)
importFrom(utils,packageVersion)
importFrom(utils,tail)
