#' Example data about mental health descirptions
#'.
#'
#' @format A data frame with 500 participants and 13 variables:
#' \describe{
#'   \item{Depselect}{Words that respondents have selected from a pre-defined list}
#'   \item{Worselect}{Words that respondents have selected from a pre-defined list}
#'   \item{Depword}{Wrods where respondents describe their experience with depression in life}
#'   \item{Worword}{Words where respondents describe their experience with depression in life}
#'   \item{Depphrase}{phrases where respondents describe their experience with depression in life}
#'   \item{Worphrase}{Phrases where respondents describe their experience with anxiety in life}
#'   \item{Deptext}{Text where respondents describe their experience with depression in life}
#'   \item{Wortext}{Text where respondents describe their experience with anxiety in life}
#'   \item{Gender}{Respondents gender 0=male, 1=female}
#'   \item{Age}{respondents age in years}
#'   \item{PHQ9tot}{total score of the respondents PHQ-9}
#'   \item{GAD7tot}{total score of the respondents GAD-7}
#'   \item{MentalSickDays}{self-reported sick days due to mental health issues}
#'   \item{MentalSickDaysYear}{self-reported sick days due to mental health issues in the last year}
#'   \item{HealthCareMental}{self-reported health care visits due to mental health}
#'   \item{SickDaysMonth}{self-reported sick days last month}
#'   \item{SickDaysYear}{self-reported sick days last year}
#'   \item{HealthCareVisits}{self-reported health care visits}
#' }
#' @source <https://osf.io/preprints/psyarxiv/p67db>
"dep_wor_data"